@@ -48,6 +48,9 @@ ZERO_OPTIMIZATION_STAGE_1 = 'stage_1'
 ZERO_OPTIMIZATION_STAGE_2 = 'stage_2'
 ZERO_OPTIMIZATION_STAGE_3 = 'stage_3'
 
+BASE_OPTIMIZER_STATE = 'base_optimizer_state'
+SINGLE_PARTITION_OF_FP32_GROUPS = "single_partition_of_fp32_groups"
+
 ZERO_OPTIMIZATION_STAGE_DEFAULT = ZERO_OPTIMIZATION_DISABLED
 
 ZERO_OPTIMIZATION_ALLGATHER_PARTITIONS = 'allgather_partitions'
@@ -74,7 +77,7 @@ ZERO_OPTIMIZATION_LOAD_FROM_FP32_WEIGHTS = 'load_from_fp32_weights'
 ZERO_OPTIMIZATION_LOAD_FROM_FP32_WEIGHTS_DEFAULT = True
 
 ZERO_OPTIMIZATION_ELASTIC_CHECKPOINT = 'elastic_checkpoint'
-ZERO_OPTIMIZATION_ELASTIC_CHECKPOINT_DEFAULT = True
+ZERO_OPTIMIZATION_ELASTIC_CHECKPOINT_DEFAULT = False
 
 ZERO_OPTIMIZATION_CPU_OFFLOAD = 'cpu_offload'
 ZERO_OPTIMIZATION_CPU_OFFLOAD_DEFAULT = False
